{
  ----------------------------------
  Filename      : REACT.PAS
  Created       : 22/02/2000
  Last modified : 09/05/2000
  Target        : Turbo Pascal 7.0
  Author        : Sven Maerivoet
 
  All rights reserved.
  Universitaire Instelling Antwerpen
  ----------------------------------
}

Program React;

Uses
  Crt, Dos, Graph, T2;

Var
  BrakePedalDistance,
  BrakePedalPosition,
  BrakePedalTreshold  : Integer;
  HStart, MStart,
  SStart, S100Start,
  ReactionTime        : Word;
  NrOfRuns, i         : Integer;
  ReactionTimes       : Array [1..255] Of LongInt;
  S                   : String;
  Dummy               : Real;

Procedure InitGraphics;
Var
  Driver, Mode, Result : Integer;
Begin
  Driver := VGA;
  Mode := VGAHi;
  InitGraph(Driver,Mode,'');
  Result := GraphResult;
  If (Result <> grOk) Then
  Begin
    WriteLn ('Error, graphics could not be initialized.');
    WriteLn ('  => ',GraphErrorMsg(Result));
    Halt;
  End;
End;

Procedure DrawScreen (NrOfRun : Integer);
Var
  Temp : String;
Begin
  ClearDevice;

  { draw the car's framework }
  SetColor (LightGray);
  Line (0,0,50,420 - 180 + 50);
  Line (50,420 - 180 + 50,50 + 24,420 - 180 + 50);
  Line (50 + 77,420 - 180 + 50,50 + 223,420 - 180 + 50);
  Line (50 + 275,420 - 180 + 50,639,420 - 180 + 50);
  Line (50,420 - 180 + 50,0,420 - 180 + 100);

  { draw the outside }
  SetColor (LightGray);
  Line (27,150,639,150);
  SetFillStyle (SolidFill,Blue);
  FloodFill (1,50,LightGray);
  SetFillStyle (SolidFill,LightBlue);
  FloodFill (300,10,LightGray);

  { draw the lanemarkers }
  SetLineStyle (DashedLn,0,ThickWidth);
  SetColor (White);
  Line (45,250,180,150);
  SetLineStyle (SolidLn,0,NormWidth);

  { draw the steering wheel }
  SetColor (LightGray);
  Circle (200,420,180);
  Circle (200,420,150);
  SetFillStyle (SolidFill,Brown);
  FloodFill (200,420 - 170,LightGray);

  { fill the car's interior }
  SetFillStyle (SolidFill,DarkGray);
  FloodFill (10,470,LightGray);
  FloodFill (200,470,LightGray);
  FloodFill (500,470,LightGray);

  { draw the car in front }
  SetFillStyle (SolidFill,Green);
  Bar (320,100,480,140);
  Bar (300,140,500,200);

  { draw it's wheels }
  SetFillStyle (SolidFill,DarkGray);
  Bar (310,201,340,240);
  Bar (460,201,490,240);

  SetTextStyle (DefaultFont,HorizDir,1);
  SetTextJustify (LeftText,CenterText);
  SetColor (Yellow);
  OutTextXY (10,10,'Mitrasim 2000 [React]');
  OutTextXY (10,30,'Sven Maerivoet, 1999 - 2000');
  SetColor (White);
  Str (NrOfRun,S);
  Str (NrOfRuns,Temp);
  OutTextXY (40,60,'Run #' + S + ' of ' + Temp);
  SetTextJustify (RightText,CenterText);
  OutTextXY (590,470 - BrakePedalDistance,'100 %');
  OutTextXY (590,470,'Brake pressure 0 %');
  OutTextXY (590,470 - BrakePedalTreshold,'Treshold');

  SetColor (Blue);
  Rectangle (0,0,639,479);
End;

Procedure DrawNoLights;
Begin
  SetFillStyle (SolidFill,Red);
  Bar (310,155,330,185);
  Bar (470,155,490,185);
End;

Procedure DrawRedLights;
Begin
  SetFillStyle (SolidFill,LightRed);
  Bar (310,155,330,185);
  Bar (470,155,490,185);
End;

Procedure DrawBreakPressure;
Begin
  SetFillStyle (SolidFill,Black);
  Bar (600,470 - BrakePedalDistance,630,470);
  SetFillStyle (SolidFill,Red);
  Bar (600,470 - BrakePedalPosition,630,470);
  SetColor (Yellow);
  Rectangle (600,470 - BrakePedalDistance,630,470);
  Line (601,470 - BrakePedalTreshold,629,470 - BrakePedalTreshold);
End;

Procedure StartChrono;
Begin
  GetTime(HStart,MStart,SStart,S100Start);
End;

Function GetChrono : Word;
Var
  H, M, S, S100 : Word;
Begin
  GetTime(H,M,S,S100);
  { ignore H en M }
  If (S < SStart) Then
    Inc (S,60);
  GetChrono := ((S * 100) + S100) - ((SStart * 100) + S100Start);
End;

Procedure MeasureReactionTime;
Var
  Ch                           : Char;
  HsToWait,
  StartHour, StartMinute,
  StartSecond, StartSec100,
  Hour, Minute, Second, Sec100 : Word;
  TempTotal, Total             : LongInt;
Begin
  SetTextStyle (DefaultFont,HorizDir,1);
  SetTextJustify (RightText,CenterText);
  SetColor (White);
  OutTextXY (630,20,'Place your rightfoot on the throttle');
  OutTextXY (630,40,'and press a key to begin...');
  Repeat Until KeyPressed;
  { preserve invariance }
  Ch := ReadKey;
  SetFillStyle (SolidFill,LightBlue);
  Bar (300,5,635,55);

  DrawNoLights;

  { wait a random time }
  HsToWait := 200 + Random(500);
  GetTime (StartHour,StartMinute,StartSecond,StartSec100);
  Total := StartSec100 +
           (StartSecond * 100);
  Repeat
    GetTime (Hour,Minute,Second,Sec100);
    TempTotal := Sec100 +
                 (Second * 100);

    T2Sample;
    BrakePedalPosition := T2Y - T2YMin;
    If (BrakePedalPosition > T2YMax) Then
      BrakePedalPosition := T2YMax;
    Sound (T2YMax - BrakePedalPosition);
    If (BrakePedalPosition < 10) Then
      BrakePedalPosition := 0;
    DrawBreakPressure;
    If (BrakePedalPosition >= BrakePedalTreshold) Then
      Sound (2000);
  Until (Abs(TempTotal - Total) >= HsToWait);

  DrawRedLights;

  StartChrono;
  Repeat
    T2Sample;
    BrakePedalPosition := T2Y - T2YMin;
    Sound (T2YMax - BrakePedalPosition);
    If (BrakePedalPosition < 10) Then
      BrakePedalPosition := 0;
    DrawBreakPressure;
    If (BrakePedalPosition >= BrakePedalTreshold) Then
      Sound (2000);
  Until (BrakePedalPosition >= BrakePedalTreshold);
  NoSound;
  ReactionTime := GetChrono;

  DrawNoLights;
End;

Procedure ShowReactionTime;
Var
  mX, mY : Integer;
  Ch     : Char;
Begin
  SetFillStyle (SolidFill,Blue);
  mX := GetMaxX Div 2;
  mY := GetMaxY Div 2;
  Bar (mX - 250,mY - 100,mX + 250,mY + 100);
  SetColor (Yellow);
  Rectangle (mX - 250,mY - 100,mX + 250,mY + 100);

  SetTextStyle (DefaultFont,HorizDir,2);
  SetTextJustify (CenterText,CenterText);
  SetColor (White);
  Dummy := ReactionTime / 100.0;
  Str (Dummy:4:2,S);
  OutTextXY (mX,mY,'Reaction time = ' + S + ' s');

  SetTextStyle (DefaultFont,HorizDir,1);
  SetColor (White);
  OutTextXY (mX,mY + 60,'Please press a key to continue...');

  Repeat Until KeyPressed;
  { preserve invariance }
  Ch := ReadKey;
End;

Procedure ShowReport;
Var
  Sum,
  i, j    : LongInt;
  Average : Word;
  StdDev  : Real;
Begin
  WriteLn ('Reaction times for test-subject :');
  WriteLn ('---------------------------------');
  WriteLn;
  For i := 1 To NrOfRuns Do
  Begin
    Dummy := ReactionTimes[i] / 100.0;
    Str (Dummy:4:2,S);
    WriteLn ('  Run #',i,' : ',S,' s');
  End;
  WriteLn;

  { calculate average }
  Sum := 0;
  For i := 1 To NrOfRuns Do
    Inc (Sum,ReactionTimes[i]);
  Average := Sum Div NrOfRuns;

  Dummy := Average / 100.0;
  Str (Dummy:4:2,S);
  WriteLn ('Average is ',S,' s.');

  Sum := 0;
  For i := 1 To NrOfRuns Do
  Begin
    j := (ReactionTimes[i] - Integer(Average)) *
         (ReactionTimes[i] - Integer(Average));
    Inc (Sum,j);
  End;
  StdDev := Sqrt(Sum / (NrOfRuns - 1));

  Dummy := StdDev / 100.0;
  Str (Dummy:4:2,S);
  WriteLn ('Standard deviation is ',S,' s.');
End;

Begin
  { test if graphics are available }
  InitGraphics;
  CloseGraph;

  WriteLn ('Mitrasim 2000 [React] - Sven Maerivoet, 1999 - 2000');
  WriteLn;

  { calibrate the Thrustmaster Formula T2 }
  T2Calibrate;

  Repeat
    Write ('Enter number of runs (2 - 255) :');
    ReadLn(NrOfRuns);
  Until ((NrOfRuns >= 2) And (NrOfRuns <= 255));

  BrakePedalDistance := T2YMax - T2YMin;
  BrakePedalTreshold := (7 * BrakePedalDistance) Div 10;

  { initialise the graphics }
  InitGraphics;
  FillChar (ReactionTimes,SizeOf(ReactionTimes),0);

  For i := 1 To NrOfRuns Do
  Begin
    DrawScreen (i);
    DrawNoLights;
    { measure reaction time }
    MeasureReactionTime;
    ShowReactionTime;
    ReactionTimes[i] := ReactionTime;
  End;

  CloseGraph;

  ShowReport;
End.
